<template>
  <LoadingLayout :loading="state.loading">
    <AppContainer>
      <ProTableQueryFilterBar @on-reset="reset">
        <template #query>
          <QueryFilterItem tip-content="配置状态">
            <FieldRadio
              v-model="extraParamState.flexEnterpriseSettingStatus"
              :value-enum="FlexEnterpriseSettingStatusText"
              buttonStyle
              showAllBtn
              @change="getList()"
            />
          </QueryFilterItem>
          <QueryFilterItem>
            <SearchInput
              v-model="extraParamState.searchWord"
              style="width: 200px"
              placeholder="企业名称/法人/联系人"
              @on-click-search="getList"
              @keyup.enter="getList()"
            >
            </SearchInput>
          </QueryFilterItem>
        </template>
        <template #btn>
          <el-button
            v-if="checkSubModuleItemShow('pageButton', 'addBtn')"
            @click="addOrEditEnterprise()"
            icon="Plus"
            type="primary"
            >新增</el-button
          >
        </template>
      </ProTableQueryFilterBar>
      <ProTableV2 v-bind="proTableProps" :columns="column" :operationBtns="operationBtns">
      </ProTableV2>
    </AppContainer>
    <ConfigureDialog v-bind="dialogProps" />
  </LoadingLayout>
</template>

<script setup lang="ts">
import {
  ProTableQueryFilterBar,
  OperationBtnType,
  ProTableV2,
  SearchInput,
  LoadingLayout,
  AppContainer,
  QueryFilterItem,
  useTable,
  useFormDialog,
  FieldRadio,
} from '@bole-core/components';
import { useAccess, useGlobalEventContext } from '@/hooks';
import * as flexEnterpriseServices from '@/services/api/FlexEnterprise';
import {
  SearchType,
  FlexEnterpriseSettingStatusText,
  FlexEnterpriseSettingStatus,
  FlexEnterpriseCertificationStatusText,
  EnterpriseConfigureType,
  MessageChannelEnum,
  VerifyStatus,
  SignChannelEnum,
  ChargeTypeEnum,
  FlexEnterpriseStatus,
} from '@/constants';
import ConfigureDialog from './components/ConfigureDialog.vue';
import { OrderInputType, Message } from '@bole-core/core';
import { useQueryClient } from '@tanstack/vue-query';
import { useSearchSettingType } from '@/hooks';

defineOptions({
  name: 'EnterpriseManageList',
});

const operationBtnMap: Record<string, OperationBtnType> = {
  editBtn: { emits: { onClick: (role) => addOrEditEnterprise(role) } },
  detailBtn: { emits: { onClick: (role) => handleDetail(role) } },
  configBtn: { emits: { onClick: (role) => openDialog(role) } },
  freezeBtn: { emits: { onClick: (role) => handleSetStatus(role) } },
};

const { checkSubModuleItemShow, column, operationBtns } = useAccess({
  operationBtnMap,
});

const eventContext = useGlobalEventContext();

eventContext.addEvent('enterprise:add', () => {
  getList();
});

eventContext.addEvent('enterprise:edit', () => {
  getList(paginationState.pageIndex);
});

const router = useRouter();

const BaseState = {
  loading: true,
};
const queryClient = useQueryClient();
const { searchSettingTypeList: typeList } = useSearchSettingType({
  searchType: SearchType.IndustryCategory,
});

const state = reactive({ ...BaseState });

onMounted(async () => {
  await getList();
  state.loading = false;
});

const {
  getDataSource: getList,
  proTableProps,
  paginationState,
  extraParamState,
  reset,
} = useTable(
  async ({ pageIndex, pageSize }, extraParamState) => {
    try {
      let params: API.GetFlexEnterpriseInput = {
        pageModel: {
          rows: pageSize,
          page: pageIndex,
          orderInput: extraParamState.orderInput,
        },
        flexEnterpriseSettingStatus: extraParamState.flexEnterpriseSettingStatus,
        searchWord: extraParamState.searchWord,
      };

      let res = await flexEnterpriseServices.getFlexEnterpriseList(params, {
        showLoading: !state.loading,
      });
      return res;
    } catch (error) {
      console.log('error: ', error);
    }
  },
  {
    defaultExtraParams: {
      searchWord: '',
      orderInput: [{ property: 'id', order: OrderInputType.Desc }],
      flexEnterpriseSettingStatus: '' as any as FlexEnterpriseSettingStatus,
    },
    queryKey: ['flexEnterpriseServices/getFlexEnterpriseList'],
    columnsRenderProps: {
      settingStatus: { type: 'enum', valueEnum: FlexEnterpriseSettingStatusText },
      certificationStatus: { type: 'enum', valueEnum: FlexEnterpriseCertificationStatusText },
    },
  }
);

function openDialog(row?: API.FlexEnterpriseDto) {
  if (row) {
    handleEdit({
      id: row.id,
      enterpriseConfigureType: EnterpriseConfigureType.Bank,
      flexEnterpirseId: row.id,
      openBank: row.flexEnterpriseBankDto?.openBank,
      openBranchBank: row.flexEnterpriseBankDto?.openBranchBank,
      bankAccount: row.flexEnterpriseBankDto?.bankAccount,
      verifyStatus: row.flexEnterpriseBankDto?.verifyStatus,
      signChannel: row.enterpriseSignSettingDto?.signChannel,
      chargeType: row.enterpriseSignSettingDto?.chargeType,
      realVerifyCost: row.enterpriseSignSettingDto?.realVerifyCost ?? 0,
      signCost: row.enterpriseSignSettingDto?.signCost ?? 0,
      mergeSignCost: row.enterpriseSignSettingDto?.mergeSignCost ?? 0,
      messageCost: row.flexEnterpriseMessageSettingDto?.messageCost ?? 0,
      messageChannel: row.flexEnterpriseMessageSettingDto?.messageChannel,
    });
  } else {
    handleAdd({
      enterpriseConfigureType: EnterpriseConfigureType.Bank,
    });
  }
}

const { dialogProps, handleAdd, handleEdit, editForm, dialogState } = useFormDialog({
  onConfirm: handleAddOrEdit,
  defaultFormParams: {
    id: '',
    flexEnterpirseId: '',
    enterpriseConfigureType: '' as any as EnterpriseConfigureType,
    openBank: '',
    openBranchBank: '',
    bankAccount: '',
    verifyStatus: '' as any as VerifyStatus,
    signChannel: '' as any as SignChannelEnum,
    chargeType: '' as any as ChargeTypeEnum,
    realVerifyCost: 0,
    signCost: 0,
    mergeSignCost: 0,
    messageCost: 0,
    messageChannel: '' as any as MessageChannelEnum,
  },
});

async function handleAddOrEdit(type: EnterpriseConfigureType) {
  try {
    let res;
    if (type === EnterpriseConfigureType.Bank) {
      res = await createOrEditFlexEnterpriseBankSetting();
    }
    if (type === EnterpriseConfigureType.Electronic) {
      res = await createOrEditFlexEnterpriseSignSetting();
    }
    if (type === EnterpriseConfigureType.ShortMessage) {
      res = await createOrEditFlexEnterpriseMessageSetting();
    }
    if (res) {
      Message.successMessage('操作成功');
      getList(paginationState.pageIndex);
      dialogState.dialogVisible = false;
    }
  } catch (error) {}
}

async function createOrEditFlexEnterpriseBankSetting() {
  try {
    let params: API.CreateOrEditFlexEnterpriseBankInput = {
      id: editForm.id,
      openBank: editForm.openBank,
      openBranchBank: editForm.openBranchBank,
      bankAccount: editForm.bankAccount,
      verifyStatus: editForm.verifyStatus,
      flexEnterpirseId: editForm.flexEnterpirseId,
    };
    return await flexEnterpriseServices.createOrEditFlexEnterpriseBankSetting(params);
  } catch (error) {}
}
async function createOrEditFlexEnterpriseSignSetting() {
  try {
    let params: API.CreateOrEditFlexEnterpriseSignSettingInput = {
      id: editForm.id,
      flexEnterpirseId: editForm.flexEnterpirseId,
      signChannel: editForm.signChannel,
      chargeType: editForm.chargeType,
      realVerifyCost: editForm.realVerifyCost,
      signCost: editForm.signCost,
      mergeSignCost: editForm.mergeSignCost,
    };
    return await flexEnterpriseServices.createOrEditFlexEnterpriseSignSetting(params);
  } catch (error) {}
}
async function createOrEditFlexEnterpriseMessageSetting() {
  try {
    let params: API.CreateOrEditFlexEnterpriseMessageSettingInput = {
      id: editForm.id,
      flexEnterpirseId: editForm.flexEnterpirseId,
      messageChannel: editForm.messageChannel,
      messageCost: editForm.messageCost,
    };
    return await flexEnterpriseServices.createOrEditFlexEnterpriseMessageSetting(params);
  } catch (error) {}
}

async function handleSetStatus(row: API.FlexEnterpriseDto) {
  try {
    let params: API.SetFlexEnterpriseStatusInput = {
      id: row.id,
      status:
        row.status === FlexEnterpriseStatus.Frozen
          ? FlexEnterpriseStatus.Normal
          : FlexEnterpriseStatus.Frozen,
    };
    let res = await flexEnterpriseServices.setFlexEnterpriseStatus(params);
    if (res) {
      Message.successMessage('操作成功');
      getList(paginationState.pageIndex);
    }
  } catch (error) {}
}

function addOrEditEnterprise(row?: API.FlexEnterpriseDto) {
  router.push({ name: 'AddOrEditEnterprise', params: { id: row?.id ?? '' } });
}
function handleDetail(row: API.FlexEnterpriseDto) {
  router.push({ name: 'EnterpriseDetail', params: { id: row?.id ?? '' } });
}
</script>
